import bpy

from mathutils import Vector


from ..types import button
from ....icons import gizmos
from ....utils import addon
from .nsuite import NsuGizmo


class NSOLVE_GGT_Nsolve(bpy.types.GizmoGroup):
    bl_idname = 'NSOLVE_GGT_Nsolve'
    bl_label = 'Nsolve Gizmo'
    bl_space_type = 'VIEW_3D'
    bl_context_mode = 'EDIT_MESH'
    bl_region_type = 'WINDOW'
    bl_options = {'SHOW_MODAL_ALL', 'SCALE'}


    @classmethod
    def poll(cls, context):
        active_tool = bpy.context.workspace.tools.from_space_view3d_mode(bpy.context.mode, create=False)
        nsolve_tool = True if active_tool and active_tool.idname == 'nsolve.nsolvetool' else False
        return context.space_data.show_gizmo_tool and context.edit_object and nsolve_tool

    def setup(self, context):
        self.gizmos.clear()

    def refresh(self, context):

        self.gizmos.clear()

        nsolve = addon.pref().tools.nsolve
        if nsolve.nso.hide:
            self.create_lmb_tab(context, nsolve)

        if nsolve.nsu.hide:
            NsuGizmo.create(self, context)

    def create_lmb_tab(self, context, nsolve):
        # Calculate adjusted size and spacing based on DPI
        scale_factor = nsolve.main_gizmo_scale
        scale = 12 * scale_factor
        base_size = Vector((scale, scale))
        ui_scale = context.preferences.system.ui_scale
        base_space = 28 * scale_factor * ui_scale

        props = addon.pref().tools.nsolve

        region = context.region
        area_width = region.width
        area_height = region.height

        # Calculate positions based on region size and DPI
        _location = Vector(props.main_gizmo_location)
        xpos = (_location.x / 100.0) * area_width
        ypos = (_location.y / 100.0) * area_height

        location = Vector((xpos, ypos))
        flip = props.main_gizmo_flip
        if (flip):
            size_bar = Vector((scale/4, scale))
        else:
            size_bar = Vector((scale, scale/4))

        # Create buttons with adjusted sizes and positions
        draw_button0 = button.Draw(self, location, size_bar)
        draw_button0.operator("nsolve.transform_gizmo", properties={'gizmo': 'NSO'})

        nsolve_gizmo = addon.pref().tools.nsolve.main.operator
        pushed_select, pushed_join, pushed_merge, pushed_slide, pushed_line, pushed_collapse, pushed_dissolve, pushed_offset, pushed_knife = False, False, False, False, False, False, False, False, False
        if nsolve_gizmo == 'SELECT':
            pushed_select = True
        elif nsolve_gizmo == 'JOIN':
            pushed_join = True
        elif nsolve_gizmo == 'MERGE':
            pushed_merge = True
        elif nsolve_gizmo == 'SLIDE':
            pushed_slide = True
        elif nsolve_gizmo == 'LINE':
            pushed_line = True
        elif nsolve_gizmo == 'COLLAPSE':
            pushed_collapse = True
        elif nsolve_gizmo == 'DISSOLVE':
            pushed_dissolve = True
        elif nsolve_gizmo == 'OFFSET':
            pushed_offset = True
        elif nsolve_gizmo == 'KNIFE':
            pushed_knife = True

        gizmo_definitions = {
            'SELECT': {
                'operator': "nsolve.set_gizmo",
                'properties': {'gizmo': 'SELECT', 'mode': 'SET', 'description': 'Select'},
                'icon': gizmos.select.icon,
                'pushed': pushed_select
            },
            'JOIN': {
                'operator': "nsolve.set_gizmo",
                'properties': {'gizmo': 'JOIN', 'mode': 'SET', 'description': 'Join'},
                'icon': gizmos.join.icon,
                'pushed': pushed_join
            },
            'MERGE': {
                'operator': "nsolve.set_gizmo",
                'properties': {'gizmo': 'MERGE', 'mode': 'SET', 'description': 'Merge'},
                'icon': gizmos.merge.icon,
                'pushed': pushed_merge
            },
            'SLIDE': {
                'operator': "nsolve.set_gizmo",
                'properties': {'gizmo': 'SLIDE', 'mode': 'SET', 'description': 'Slide'},
                'icon': gizmos.slide.icon,
                'pushed': pushed_slide
            },
            'LINE': {
                'operator': "nsolve.set_gizmo",
                'properties': {'gizmo': 'LINE', 'mode': 'SET', 'description': 'Line'},
                'icon': gizmos.project_line.icon,
                'pushed': pushed_line
            },
            'COLLAPSE': {
                'operator': "nsolve.set_gizmo",
                'properties': {'gizmo': 'COLLAPSE', 'mode': 'SET', 'description': 'Collapse'},
                'icon': gizmos.collapse_verts.icon,
                'pushed': pushed_collapse
            },
            'DISSOLVE': {
                'operator': "nsolve.set_gizmo",
                'properties': {'gizmo': 'DISSOLVE', 'mode': 'SET', 'description': 'Dissolve'},
                'icon': gizmos.dissolve.icon,
                'pushed': pushed_dissolve
            },
            'OFFSET': {
                'operator': "nsolve.set_gizmo",
                'properties': {'gizmo': 'OFFSET', 'mode': 'SET', 'description': 'Offset'},
                'icon': gizmos.offset_edges.icon,
                'pushed': pushed_offset
            },
            'KNIFE': {
                'operator': "nsolve.set_gizmo",
                'properties': {'gizmo': 'KNIFE', 'mode': 'SET', 'description': 'Knife'},
                'icon': gizmos.knife.icon,
                'pushed': pushed_knife
            }
        }

        for item in props.main_gizmo_list_items:
            gizmo_info = gizmo_definitions.get(item.operator)
            if gizmo_info:
                location = location.copy()
                if flip:
                    location.x += base_space
                else:
                    location.y -= base_space
                draw_button = button.Draw(self, location, base_size, gizmo_info['icon'], gizmo_info['pushed'])
                draw_button.operator(gizmo_info['operator'], properties=gizmo_info['properties'])
