import bpy
import bmesh

from ....utils.nmesh import Nmesh


class NSOLVE_OT_SlideVert(bpy.types.Operator):
    bl_idname = "nsolve.slide_vert"
    bl_label = "Slide Vert"
    bl_description = f"{bl_label}\n\nSlide vertex along the edge"
    bl_options = {'REGISTER', 'UNDO', 'BLOCKING', 'GRAB_CURSOR', 'INTERNAL'}

    bm: bmesh.types.BMesh = None
    nm: Nmesh = None

    @classmethod 
    def poll(cls, context):
        return context.area.type == 'VIEW_3D' and context.mode == 'EDIT_MESH'

    def invoke(self, context, event):
        obj = context.edit_object
        bm = bmesh.from_edit_mesh(obj.data)
        nm = Nmesh.from_bmesh(context, bm, obj)

        select_mode = context.tool_settings.mesh_select_mode[:]
        if select_mode[1]:
            if not select_mode[0]:

                nm.detect_element(event, mode=(False, True, False))
                if nm.element[0] == 'EDGE':
                    edge = nm.element[1]

                    bpy.ops.mesh.select_all(action='DESELECT')
                    edge.select = True

                    bpy.ops.transform.edge_slide('INVOKE_DEFAULT')

                    return {'FINISHED'}

        nm.detect_element(event, mode=(True, False, False))

        vert = nm.element[1]

        if not vert:
            return {'CANCELLED'}

        bpy.ops.mesh.select_all(action='DESELECT')
        vert.select = True

        bpy.ops.transform.vert_slide('INVOKE_DEFAULT')

        return {'FINISHED'}