import bpy
from . import ops, tools, types
from .. import __package__ as base_package


class NSolve_Preference(bpy.types.AddonPreferences):
    bl_idname = base_package

    settings: bpy.props.EnumProperty(
        name='Settings',
        description='Settings to display',
        items=[
            ('OPTIONS', 'Options', ''),
            ('THEME', 'Theme', ''),
            ('NFLOW', 'nFlow', ''),
        ],
        default='OPTIONS'
    )

    tools: bpy.props.PointerProperty(type=tools.types.pref)
    theme: bpy.props.PointerProperty(type=types.Theme)

    nflow: bpy.props.PointerProperty(type=ops.nflow.types.preferences)
    menus: bpy.props.BoolProperty(default=False)
    menus_type: bpy.props.EnumProperty(default='UNIFIED', items=[('UNIFIED', 'Unified', ''), ('SEPARATED', 'Separated', '')])

    def draw(self, context):
        layout = self.layout
        column = layout.column(align=True)
        split = column.split(factor=0.2)
        col = split.column(align=True)
        col.prop(self, 'settings', expand=True)
        col = split.column(align=True)
        col.use_property_split = True

        if self.settings == 'OPTIONS':

            pref = context.preferences.addons[base_package].preferences
            col = col.column(align=True)
            col.prop(pref, 'menus', text='Menus')
            col.prop(pref, 'menus_type', text='Menu Type')

            self.transform(context, col, pref)

        elif self.settings == 'THEME':
            flow = col.grid_flow(row_major=False, columns=0, even_columns=True, even_rows=False, align=False)
            theme = context.preferences.addons[base_package].preferences.theme

            self.theme_draw(flow, theme.ops.mesh.project.circle, 'Project Circle')
            self.theme_draw(flow, theme.ops.mesh.project.grid, 'Project Grid')
            self.theme_draw(flow, theme.ops.mesh.project.line, 'Project Line')
            self.theme_draw(flow, theme.ops.mesh.project.loop, 'Project Loop')
            self.theme_draw(flow, theme.ops.mesh.bevel, 'Bevel')
            self.theme_draw(flow, theme.ops.mesh.collapse_verts, 'Collapse Verts')
            self.theme_draw(flow, theme.ops.mesh.cut, 'Cut')
            self.theme_draw(flow, theme.ops.mesh.dissolve, 'Dissolve')
            self.theme_draw(flow, theme.ops.mesh.hard_edge_slide, 'Hard Edge Slide')
            self.theme_draw(flow, theme.ops.mesh.hard_loop_cut, 'Hard Loop Cut')
            self.theme_draw(flow, theme.ops.mesh.join_verts, 'Join Verts')
            self.theme_draw(flow, theme.ops.mesh.merge_verts, 'Merge Verts')
            self.theme_draw(flow, theme.ops.mesh.offset_edges, 'Offset Edges')
            self.theme_draw(flow, theme.ops.mesh.analyze, 'Analyze')
            self.theme_draw(flow, theme.ops.ndraw.pencil, 'Pencil')


        elif self.settings == 'NFLOW':
            self.nflow.draw(col, context)

    def theme_draw(self, layout, theme, label):
        box = layout.box()
        box.label(text=label)
        for prop in theme.bl_rna.properties:
            # ignore blender defined
            if prop.identifier == 'name' or prop.identifier == 'rna_type' : continue

            box.prop(theme, prop.identifier)

    def transform(self, context, col, pref):

        col.separator()
        nso = pref.tools.nsolve
        col = col.column()
        col.prop(nso, 'main_gizmo_location', text='NSO Loc')
        col.prop(nso, 'main_gizmo_scale', text='NSO Scale')
        col.prop(nso, 'main_gizmo_flip', text='NSO Flip')
        col.separator()

        nsu = pref.tools.nsuite
        col = col.column()
        col.prop(nsu, 'ops_gizmo_location', text='NSU Loc')
        col.prop(nsu, 'ops_gizmo_scale', text='NSU Scale')
        col.prop(nsu, 'ops_gizmo_flip', text='NSU Flip')
        col.separator()


classes = (
    NSolve_Preference,
)
